// OpenLiveVideoDoc.cpp : implementation of the COpenLiveVideoDoc class
//
#include "stdafx.h"

#include "MainFrm.h"
#include "OpenLiveVideo.h"
#include "OpenLiveVideoDoc.h"

#include "VideoManager.h"
#include "VideoThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


/********************************************************************************/
extern CToolBar   *mainToolBar;
extern CStatusBar *mainStatusBar;
/********************************************************************************/
IMPLEMENT_DYNCREATE(COpenLiveVideoDoc, CDocument)	// COpenLiveVideoDoc
/********************************************************************************/
BEGIN_MESSAGE_MAP(COpenLiveVideoDoc, CDocument)
	ON_COMMAND_RANGE(ID_VIDEODEVICE0, ID_VIDEODEVICE3, OnOpenVideoDevice)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIDEODEVICE0, ID_VIDEODEVICE3, OnUpdateOpenVideoDevice)
END_MESSAGE_MAP()
/********************************************************************************/
COpenLiveVideoDoc::COpenLiveVideoDoc()	// COpenLiveVideoDoc construction/destruction
{
	_videoThread  = NULL;
	_videoManager = NULL;

}
COpenLiveVideoDoc::~COpenLiveVideoDoc()
{
	SAFE_DELETE(_videoManager);
}
/********************************************************************************/
void COpenLiveVideoDoc::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}
/********************************************************************************/
#ifdef _DEBUG	// Generated message map functions
void COpenLiveVideoDoc::AssertValid() const { CDocument::AssertValid(); }
void COpenLiveVideoDoc::Dump(CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
/********************************************************************************/

/************************************************************************************************/
void COpenLiveVideoDoc::RefreshToolBar()
{
	if(mainToolBar)
		mainToolBar->OnUpdateCmdUI((CMainFrame*)(AfxGetApp()->GetMainWnd()), true);

	if(mainStatusBar)
		mainStatusBar->OnUpdateCmdUI((CMainFrame*)(AfxGetApp()->GetMainWnd()), true);
}
void COpenLiveVideoDoc::RecalcLayout()
{
	CFrameWnd *frame = (CFrameWnd*)AfxGetApp()->m_pMainWnd;
	
	if(frame)
		frame->RecalcLayout(TRUE);
}
/************************************************************************************************/
void COpenLiveVideoDoc::InitVideoManager()
{
	SAFE_DELETE(_videoManager);
	_videoManager = new CVideoManager();

	_videoManager->InitVideoStream((SourceDlg)-1, 0);
	StartVideoThread();

	CMenu* menu = (CMenu*)((CMainFrame*)(AfxGetApp()->GetMainWnd()))->GetMenu();
	if(_videoManager && menu)
	{
		int videoIndex = (ID_VIDEODEVICE3 - ID_VIDEODEVICE0) + 1;
		int videoDeviceNum = _videoManager->GetMaxVChannelNumber();
		for(int i = 0; i < videoIndex; i++)
		{
			CString deviceName;
			if(i < videoDeviceNum)
			{
				deviceName = _videoManager->GetVideoDeviceName(i);
				menu->ModifyMenuA(ID_VIDEODEVICE0 + i, MF_BYCOMMAND, ID_VIDEODEVICE0 + i, deviceName);
			}
			else
				menu->DeleteMenu(ID_VIDEODEVICE0 + i, MF_BYCOMMAND);
		}
	}
}
void COpenLiveVideoDoc::OnOpenVideoDevice(UINT nID)
{
	if(!_videoManager)
		return;

	int videoDeviceIndex = -1;

	switch(nID)
	{
		case ID_VIDEODEVICE0: { videoDeviceIndex = 0; } break;
		case ID_VIDEODEVICE1: { videoDeviceIndex = 1; } break;
		case ID_VIDEODEVICE2: { videoDeviceIndex = 2; } break;
		case ID_VIDEODEVICE3: { videoDeviceIndex = 3; } break;
		default: return;
	}

	if(_videoManager->GetVideoChannelNumber() == videoDeviceIndex)
		return;

	/*****************************************************/
	StopVideoThread();

	_videoManager->InitVideoStream((SourceDlg)0, videoDeviceIndex);

	StartVideoThread();
	/*****************************************************/

	RefreshToolBar(); 

	if(_viewWnd)
		PostMessage(_viewWnd->GetSafeHwnd(), WM_SIZING, 0, 0);
}
void COpenLiveVideoDoc::OnUpdateOpenVideoDevice(CCmdUI *pCmdUI)
{
	CMenu* menu = (CMenu*)((CMainFrame*)(AfxGetApp()->GetMainWnd()))->GetMenu();
	int	videoIndex = (ID_VIDEODEVICE3 - ID_VIDEODEVICE0) + 1;

	if(!menu || !_videoManager)
		return;

	for(int i = 0; i < videoIndex; i++)
		menu->CheckMenuItem(ID_VIDEODEVICE0 + i, MF_UNCHECKED);

	menu->CheckMenuItem(ID_VIDEODEVICE0 + _videoManager->GetVideoChannelNumber(), MF_CHECKED);
}
/********************************************************************************/
void COpenLiveVideoDoc::StartVideoThread()
{
	if(_videoThread)
		return;

	_videoThread = AfxBeginThread(VideoFrameThread, this, THREAD_PRIORITY_NORMAL);
}
void COpenLiveVideoDoc::StopVideoThread()
{
	if(!_videoThread)
		return;

	_killNextFrame.ResetEvent();
	_killNextFrame.SetEvent();

	::WaitForSingleObject(_nextFrameEnd.m_hObject, INFINITE); 

	_videoThread = NULL;
}
/********************************************************************************/