// OpenLiveVideoView.cpp : implementation of the COpenLiveVideoView class
//

#include "stdafx.h"
#include "OpenLiveVideo.h"

#include "OpenLiveVideoDoc.h"
#include "OpenLiveVideoDoc.h"
#include "OpenLiveVideoView.h"

#include "MemoryDC.h"
#include "VideoManager.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/********************************************************************************/
IMPLEMENT_DYNCREATE(COpenLiveVideoView, CView)	// COpenLiveVideoView
/********************************************************************************/
BEGIN_MESSAGE_MAP(COpenLiveVideoView, CView)
END_MESSAGE_MAP()
/********************************************************************************/
COpenLiveVideoView::COpenLiveVideoView()	// COpenLiveVideoView construction/destruction
{
	_memoryDC = NULL;

	_gdiplusToken = 0;

	_viewSize.SetSize(0, 0);

	Gdiplus::GdiplusStartupInput gdiplusStartupInput;
	Gdiplus::GdiplusStartup(&_gdiplusToken, &gdiplusStartupInput, NULL);
}

COpenLiveVideoView::~COpenLiveVideoView()
{
	GdiplusShutdown(_gdiplusToken);

	COpenLiveVideoDoc* pDoc = GetDocument();
	pDoc->StopVideoThread();

	SAFE_DELETE(_memoryDC);
}
/********************************************************************************/
void COpenLiveVideoView::DrawNextFrameDC(CDC* pDC)
{
	CRect clipBoxRect,oldClipBoxRect;

	clipBoxRect.SetRectEmpty();
	oldClipBoxRect.SetRectEmpty(); 

	if(!_memoryDC)
		_memoryDC = new CMemoryDC(pDC);

	_memoryDC->GetClipBox(&oldClipBoxRect);
	GetClientRect(&clipBoxRect);

	if(oldClipBoxRect != clipBoxRect)
	{
		SAFE_DELETE(_memoryDC);
		_memoryDC = new CMemoryDC(pDC, &clipBoxRect);
	}

	_memoryDC->NextFrame(pDC);
}
void COpenLiveVideoView::OnDraw(CDC* pDC)	// COpenLiveVideoView drawing
{
	COpenLiveVideoDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	CSize imageSize;
	UCHAR *data = NULL;
	BITMAPINFO *header = NULL;
	Bitmap *imageToShow = NULL;

	DrawNextFrameDC(pDC);

	if(!_memoryDC)
		return;

	Graphics graphDevice(_memoryDC->GetSafeHdc());
	graphDevice.Clear(Color(255,255,255));

	imageSize = pDoc->_videoManager->GetColorImageSize();
	header    = pDoc->_videoManager->GetColorHeader();
	data	  = pDoc->_videoManager->GetColorData();

	imageToShow = Bitmap::FromBITMAPINFO(header, data);
	graphDevice.DrawImage(imageToShow, 10, 10, imageSize.cx, imageSize.cy);

	_memoryDC->Display();

	graphDevice.ReleaseHDC(_memoryDC->GetSafeHdc());
}
/********************************************************************************/
#ifdef _DEBUG	// COpenLiveVideoView diagnostics
void COpenLiveVideoView::AssertValid() const { CView::AssertValid(); }
void COpenLiveVideoView::Dump(CDumpContext& dc) const { CView::Dump(dc); }
COpenLiveVideoDoc* COpenLiveVideoView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenLiveVideoDoc)));
	return (COpenLiveVideoDoc*)m_pDocument;
}
#endif //_DEBUG
/********************************************************************************/
void COpenLiveVideoView::OnInitialUpdate()	// COpenLiveVideoView message handlers
{
	CView::OnInitialUpdate();

	COpenLiveVideoDoc* pDoc = GetDocument();
	if(!pDoc)
		return;

	pDoc->SetViewWnd(this);
	pDoc->InitVideoManager();

	_viewSize = pDoc->_videoManager->GetColorImageSize();
}
/********************************************************************************/