#include "StdAfx.h"

#include "VideoManager.h"

/**********************************************************************/
CVideoManager::CVideoManager(void)
{
	_colorImageData = NULL;
	_grayImageData  = NULL;
	_colorHeader    = NULL;
	_grayHeader     = NULL;

	_isVideoStreamOpen = false;

	_videoFileName.Empty();

	_colorImageSize.width        = 0; 
	_colorImageSize.height       = 0; 
	_colorImageSize.colorChannel = 0;

	_grayImageSize.width         = 0; 
	_grayImageSize.height        = 0;
	_grayImageSize.colorChannel  = 0;
}
CVideoManager::~CVideoManager(void)
{
	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();
}
/**********************************************************************/
bool CVideoManager::OpenVideoFile(CString &fileName)
{
	bool startResult = false;
	BOOL openStatus = FALSE;

	_isVideoStreamOpen = false;

	if(_videoModulePtr.GetInterfacePtr())
		_videoModulePtr.Release();

	try
	{
		_videoModulePtr = IVideoModulePtr(__uuidof(VideoModule));
	}
	catch(_com_error err)
		{
			AfxMessageBox(err.ErrorMessage()); 
			return false;
		}
      
	_videoModulePtr->OpenVideoFile(_bstr_t(fileName));    
	openStatus = _videoModulePtr->IsVideoStreamOpen(); 

	if(!openStatus)
		return false;

	_isVideoStreamOpen  = true;

	_colorHeader = (BITMAPINFO*)_videoModulePtr->GetColorHeader();
	_grayHeader  = (BITMAPINFO*)_videoModulePtr->GetGrayHeader();

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	return _isVideoStreamOpen;
}
bool CVideoManager::StopVideoStream()
{
	if(!_videoModulePtr.GetInterfacePtr())
		return false;

	_videoModulePtr->StopVideoStream();
	_videoModulePtr.Release(); 

	return true;
}
/**********************************************************************/
int CVideoManager::NextFrame()
{
	int result = -1;

	if(!_videoModulePtr.GetInterfacePtr())
		return result;

	_videoModulePtr->NextFrame();
	result = _videoModulePtr->GetVideoStatus();

	if(result != 1)
		return result;

	_colorImageSize = _videoModulePtr->GetColorVideoImageSize();
	_grayImageSize  = _videoModulePtr->GetGrayVideoImageSize();

	_colorImageData = _videoModulePtr->GetColorVideoData();
	_grayImageData  = _videoModulePtr->GetGrayVideoData();

	return result;
}
/**********************************************************************/