// SaveLiveVideoDoc.cpp : implementation of the CSaveLiveVideoDoc class
//
#include "stdafx.h"

#include "MainFrm.h"
#include "SaveLiveVideo.h"
#include "SaveLiveVideoDoc.h"

#include "VideoManager.h"
#include "VideoThread.h"

#include "CompressorDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


/********************************************************************************/
extern CToolBar   *mainToolBar;
extern CStatusBar *mainStatusBar;
/********************************************************************************/
IMPLEMENT_DYNCREATE(CSaveLiveVideoDoc, CDocument)	// CSaveLiveVideoDoc
/********************************************************************************/
BEGIN_MESSAGE_MAP(CSaveLiveVideoDoc, CDocument)
	ON_COMMAND_RANGE(ID_VIDEODEVICE0, ID_VIDEODEVICE3, OnOpenVideoDevice)
	ON_UPDATE_COMMAND_UI_RANGE(ID_VIDEODEVICE0, ID_VIDEODEVICE3, OnUpdateOpenVideoDevice)

	ON_COMMAND(ID_FILE_STARTRECORD, OnStartVideoRecord)
	ON_COMMAND(ID_FILE_STOPRECORD,  OnStopVideoRecord)
	ON_UPDATE_COMMAND_UI(ID_FILE_STARTRECORD, OnUpdateStartVideoRecord)
	ON_UPDATE_COMMAND_UI(ID_FILE_STOPRECORD,  OnUpdateStopVideoRecord)
	ON_COMMAND(ID_FILE_CHOOSEVIDEOCOMPRESSOR, &CSaveLiveVideoDoc::OnChooseVideoCompressor)
END_MESSAGE_MAP()
/********************************************************************************/
CSaveLiveVideoDoc::CSaveLiveVideoDoc()	// CSaveLiveVideoDoc construction/destruction
{
	_videoThread  = NULL;
	_videoManager = NULL;

}
CSaveLiveVideoDoc::~CSaveLiveVideoDoc()
{
	SAFE_DELETE(_videoManager);
}
/********************************************************************************/
void CSaveLiveVideoDoc::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}
/********************************************************************************/
#ifdef _DEBUG	// Generated message map functions
void CSaveLiveVideoDoc::AssertValid() const { CDocument::AssertValid(); }
void CSaveLiveVideoDoc::Dump(CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
/********************************************************************************/

/************************************************************************************************/
void CSaveLiveVideoDoc::RefreshToolBar()
{
	if(mainToolBar)
		mainToolBar->OnUpdateCmdUI((CMainFrame*)(AfxGetApp()->GetMainWnd()), true);

	if(mainStatusBar)
		mainStatusBar->OnUpdateCmdUI((CMainFrame*)(AfxGetApp()->GetMainWnd()), true);
}
void CSaveLiveVideoDoc::RecalcLayout()
{
	CFrameWnd *frame = (CFrameWnd*)AfxGetApp()->m_pMainWnd;
	
	if(frame)
		frame->RecalcLayout(TRUE);
}
/************************************************************************************************/
void CSaveLiveVideoDoc::InitVideoManager()
{
	SAFE_DELETE(_videoManager);
	_videoManager = new CVideoManager();

	_videoManager->InitVideoStream((SourceDlg)-1, 0);
	StartVideoThread();

	CMenu* menu = (CMenu*)((CMainFrame*)(AfxGetApp()->GetMainWnd()))->GetMenu();
	if(_videoManager && menu)
	{
		int videoIndex = (ID_VIDEODEVICE3 - ID_VIDEODEVICE0) + 1;
		int videoDeviceNum = _videoManager->GetMaxVChannelNumber();
		for(int i = 0; i < videoIndex; i++)
		{
			CString deviceName;
			if(i < videoDeviceNum)
			{
				deviceName = _videoManager->GetVideoDeviceName(i);
				menu->ModifyMenuA(ID_VIDEODEVICE0 + i, MF_BYCOMMAND, ID_VIDEODEVICE0 + i, deviceName);
			}
			else
				menu->DeleteMenu(ID_VIDEODEVICE0 + i, MF_BYCOMMAND);
		}
	}
}
void CSaveLiveVideoDoc::OnOpenVideoDevice(UINT nID)
{
	if(!_videoManager)
		return;

	int videoDeviceIndex = -1;

	switch(nID)
	{
		case ID_VIDEODEVICE0: { videoDeviceIndex = 0; } break;
		case ID_VIDEODEVICE1: { videoDeviceIndex = 1; } break;
		case ID_VIDEODEVICE2: { videoDeviceIndex = 2; } break;
		case ID_VIDEODEVICE3: { videoDeviceIndex = 3; } break;
		default: return;
	}

	if(_videoManager->GetVideoChannelNumber() == videoDeviceIndex)
		return;

	/*****************************************************/
	StopVideoThread();

	_videoManager->InitVideoStream((SourceDlg)0, videoDeviceIndex);

	StartVideoThread();
	/*****************************************************/

	RefreshToolBar(); 

	if(_viewWnd)
		PostMessage(_viewWnd->GetSafeHwnd(), WM_SIZING, 0, 0);
}
void CSaveLiveVideoDoc::OnUpdateOpenVideoDevice(CCmdUI *pCmdUI)
{
	CMenu* menu = (CMenu*)((CMainFrame*)(AfxGetApp()->GetMainWnd()))->GetMenu();
	int	videoIndex = (ID_VIDEODEVICE3 - ID_VIDEODEVICE0) + 1;

	if(!menu || !_videoManager)
		return;

	for(int i = 0; i < videoIndex; i++)
		menu->CheckMenuItem(ID_VIDEODEVICE0 + i, MF_UNCHECKED);

	menu->CheckMenuItem(ID_VIDEODEVICE0 + _videoManager->GetVideoChannelNumber(), MF_CHECKED);
}
/********************************************************************************/
void CSaveLiveVideoDoc::StartVideoThread()
{
	if(_videoThread)
		return;

	_videoThread = AfxBeginThread(VideoFrameThread, this, THREAD_PRIORITY_NORMAL);
}
void CSaveLiveVideoDoc::StopVideoThread()
{
	if(!_videoThread)
		return;

	_killNextFrame.ResetEvent();
	_killNextFrame.SetEvent();

	::WaitForSingleObject(_nextFrameEnd.m_hObject, INFINITE); 

	_videoThread = NULL;
}
/********************************************************************************/
void CSaveLiveVideoDoc::OnStartVideoRecord()
{
	if(!_videoManager)
		return;

	_videoManager->StartVideoRecord();
}
void CSaveLiveVideoDoc::OnStopVideoRecord()
{
	if(!_videoManager)
		return;

	_videoManager->StopVideoRecord();
}
void CSaveLiveVideoDoc::OnUpdateStartVideoRecord(CCmdUI *pCmdUI)
{
	bool enable = (_videoManager && _videoManager->IsVideoStreamOpen() && !_videoManager->IsVideoStreamRecord());
	pCmdUI->Enable(enable);
}
void CSaveLiveVideoDoc::OnUpdateStopVideoRecord(CCmdUI *pCmdUI)
{
	bool enable = (_videoManager && _videoManager->IsVideoStreamOpen() && _videoManager->IsVideoStreamRecord());
	pCmdUI->Enable(enable);
}
void CSaveLiveVideoDoc::OnChooseVideoCompressor()
{
	CCompressorDlg listDlg;

	if(!_videoManager)
		return;

	listDlg.SetVideoManager(_videoManager);
	INT_PTR result = listDlg.DoModal();

	if(result != IDOK)
		return;

	_videoManager->SetCompressoID(listDlg._compressorID);
}
